/*jslint browser:true, confusion:true, debug:true, devel:true, nomen:true, plusplus:true, vars:true */
(function(g){
	"use strict";
// try{
	
	var ax = g.ax,	// require('ax');
        xx = g.xx,
		app = g.app,
		_DEBUG = g._DEBUG || false;
		

	function validateResponse(result) {

		if(result.status >= 200 && result.status < 300) {
			//var data = JSON.parse(result.data);	// XXX: JSON.parse ignore array data 
			var data = ax.util.decodeJSON(result.data);
			
			if (data && typeof data.meta !== 'undefined' && typeof data.meta.code !== 'undefined') {
                if (typeof data.meta.code !== 'undefined' && data.meta.code === app.def.API_SUCCESS_CODE) {
                    return data.data;
                } else {
                    throw {code: data.meta.code, message: data.meta.message};
                }
			} else {
                if (_DEBUG) {
                    console.log('서버데이터 확인 필요!!!', ax.util.dump(result));
                }
                throw {code: '000', message: '서버데이터 확인 필요!!!'};
			}
		} else {
			if(_DEBUG) { console.log('validateResponse err', ax.util.dump(result)); }
			throw {code: result.status, message: 'http status error'};
		}
	}

	function appNotice(){
		
		if (app.common.isNetworkError === true) { return; }
		
		var checkCallback = false;
		var driveinfo_eb = function(e) {
			if (checkCallback) {	//XXX: 왜 success콜백이후 에러콜백이 또 떨어질까??
				if(_DEBUG) { console.log('왜 success콜백이후 에러콜백이 또 떨어질까??', e.message); }
				return; 
			}	
			console.info('>>>>>>>>>> GOOD DAY TO DIE <<<<<<<<<<', e.message);
			//'네트워크 상태가 원활치 않아 프로그램을 시작할 수 없습니다.'
			var msg = app.nls.get('svc_network_error');//$NLS$
			ax.ext.ui.alert(function() {
				try {
					ax.ext.ios.finish();
				} catch(e) { 
					ax.ext.android.finish();
				}
			}, msg, {'positive': app.nls.get('common_ok')});
		};
		
		var driveinfo_cb = function(o) {
			
			var notice,
				lang = app.nls.lang,
				currentTime,
				noticeCheck = app.common.getPreference('noticeNumber') || 0, 
				opts;
				
			try {
				notice = app.api.validateResponse(o);
				currentTime = ax.util.decodeJSON(o.data).meta.callTime;
			} catch (e) {
				driveinfo_eb(e);
				return;
			}
			checkCallback = true;
			
			if(_DEBUG) { console.info('app notice', ax.util.dump(o), 'noticeCheck', noticeCheck); }

            if (notice === null) { return; }    // 공지사항이 없는 경우

			if (lang === 'ko') {
				lang = 'content';
			} else if (lang === 'ja') {
				lang = 'contentsJa';
			} else if (lang === 'zh') {
				lang = 'contentsCha';
			} else {
				lang = 'contentEn';
			}
			
			if (currentTime < notice.startDate || currentTime > notice.endDate) { return; }
			if (notice[lang] === '') { return; }
			
			switch (notice.type) {
				case 'N': // 일반공지 
					if (noticeCheck === notice.noticeNo) { return; }
					app.common.setPreference('noticeNumber', notice.noticeNo);
					if (notice.alertType === '01') { // 닫기 / 자세히보기
					if (notice.appUrl === '') { return; }
					
					opts = { positive:app.nls.get('common_more_detail'), negative:app.nls.get('common_close') };
					ax.ext.ui.confirm(function(res){
							if (res) {
								ax.ext.ui.open(notice.appUrl);
							}
						},notice[lang], opts);
					} else if (notice.alertType === '02') { // 자세히 보기
						if (notice.appUrl === '') { return; }
						
						opts = { positive:app.nls.get('common_more_detail') };
						ax.ext.ui.alert(function(){
								ax.ext.ui.open(notice.appUrl);
							},notice[lang], opts);
					} else if (notice.alertType === '03') { // 확인
						opts = { positive:app.nls.get('common_ok') };
						ax.ext.ui.alert(function(){},notice[lang], opts);
					}
				break;
				case 'U':	// 업데이트
					if (notice.appUrl === '') { return; }
					
					var updateCheckTTL = app.common.getPreference('updateCheckTTL') || 0,
						currentTimeForCheck = new Date().getTime();
					updateCheckTTL = parseInt(updateCheckTTL, 10);
					
					//console.log(updateCheckTTL, currentTimeForCheck);
					if (updateCheckTTL > currentTimeForCheck) { return; }	// 업데이트 공지 닫기버튼 눌렀을때 7일 시간체크
					//_kmq.push(['record', 'Update alert']);
					
					opts = {'positive': app.nls.get('review_alert_btn_ok'), 'negative': app.nls.get('review_alert_btn_cancel')};
					ax.ext.ui.confirm(function(res){
						if (res) {
							//_kmq.push(['record', 'Yes_update']);
							
							updateCheckTTL = currentTimeForCheck + 10000;
							app.common.setPreference('updateCheckTTL', updateCheckTTL);
							
							if (xx.page.platform === 'android') {
								app.common.showReviewMarketList('update');
							} else {
								ax.ext.ui.open(notice.appUrl, '_blank');
							}
						} else {
							//_kmq.push(['record', 'No_update']);
							
							updateCheckTTL = currentTimeForCheck + 4000; //(86400 * 1000 * 7);
							app.common.setPreference('updateCheckTTL', updateCheckTTL);
						}
					}, notice[lang], opts);
				break;
				case 'S':	// 시스템 공지
					opts = { positive:app.nls.get('common_more_detail'), negative:app.nls.get('common_ok') };
					ax.ext.ui.confirm(function(res) {
						if (res) {
							ax.ext.ui.open(notice.appUrl);
						}
						
						try {
							ax.ext.ios.finish();
						} catch(e) {
						
							ax.ext.ga.stopTracker();
							if(_DEBUG) { console.log('stopTracker'); }
							
							setTimeout(function(){
								ax.ext.android.finish();
							}, 300);
						
						}
					},notice[lang], opts);
				break;
			}	// switch
		
			console.info('>>>>>>>>>> READY FOR BATTLE <<<<<<<<<<');
		};
		
	
		if(_DEBUG) { console.log('API_NOTICE', app.def.API_NOTICE); }
		ax.ext.net.curl({
			method:'GET',
			url: app.def.API_NOTICE,
			headers: app.def.API_HEADERS,
			timeout: app.def.TIMEOUT_API_DRIVEINFO,
			success:driveinfo_cb,
			error:driveinfo_eb
		});

	}
	
	
	//var API_BATTLE = ' http://dev.pudding.kr/battle/whois.json';
	//?svckey=face&mode=random&fileUrl=http://uccfs.paran.com/PUD/fa/ce/FACE/STAR_457/457_93998_130.jpg&featureId=16088&whoId=475&imgId=1&featurelist=TEST&imgId=0&featurelist=TEST'
	
	function battle(star, player, scb, ecb, mode) {
		
		if(_DEBUG) { console.log('BATTLE!!!', star, player); }
		
		var params = {
			starCategory: app.common.getPreference('starCategory'),
			mode: mode,
			fileUrl: star.fileUrl,
			featureId: star.featureId,
			whoId: star.whoId,
			deviceToken: app.common.getPreference('kpnsToken')
		};
		
		var url = app.def.API_BATTLE_REQUEST + '?' + $.param(params),
			i, 
			len;
		
		for (i=0, len=player.features.length; i<len; i++) {
			url += '&imgId=' + i + '&featureList=' + player.features[i];
		}

		if(_DEBUG) { console.log('API_BATTLE_REQUEST url', url); }
		
		var callback = function(result) {
			try {
				var data = app.api.validateResponse(result);
				if(_DEBUG) { console.log('API_BATTLE_REQUEST SUCCESS', data); }
				scb(data);
			} catch(e) {
				//alert('error server message ' + result.meta.code + ' ' + result.meta.msg);
				ecb({code:'999', message: result});
			}
		};
		
		var errback = function(e) { 
			if(_DEBUG) { console.log('API_BATTLE_REQUEST ERR', e.message); }
			ecb(e); 
		};
		
		ax.ext.net.curl({
			method:'GET',
			url: url,
			headers: app.def.API_HEADERS,
			timeout: app.def.TIMEOUT_API_DEFAULT,
			success: callback,
			error: errback
		});

	}


	function whois(url, scb, ecb) {
		if(_DEBUG) { console.log(url); }
		
		var callback = function(result) {
			try {
				var data = app.api.validateResponse(result);
				
				if (data.sameinfo) {
					scb(data);
				} else {
					ecb({code:'999', message: 'whois 에서 code는 success이나 sameInfo 정보가 없음!!!!'});
				}
			} catch(e) {
				ecb({code:'999', message: result});
			}
		};
		
		var errback = function(e) {
			ecb(e);
		};
		
		ax.ext.net.curl({
			method:'GET',
			url: url,
			headers: app.def.API_HEADERS,
			timeout: app.def.TIMEOUT_API_DEFAULT,
			success: callback,
			error: errback
		});
	}
	
	
	function upload(targetFile, scb, ecb) {
		var params = {
				svckey: app.def.SVC_KEY,
				v: app.def.APP_VERSION
			},
			files = {
				upFile: targetFile
			};
			
		var errback = function(e) {
			if (_DEBUG) { console.log('API_UPLOAD ERR', e.message); }
			if (scb) { scb(e); }
		};
		
		var callback = function(o) {
			try {
				
				if (_DEBUG) { console.log('API_UPLOAD SUCCESS', o.data); }
				var data = app.api.validateResponse(o);
				if (scb) { scb(o); }
			} catch(e) {
				errback('validateResponse err' + e.message);
			}
		};
	
		if (_DEBUG) { console.log('API_UPLOAD url', app.def.API_UPLOAD); }
		
		var opts = {
			url: app.def.API_UPLOAD,
			headers: app.def.API_HEADERS,
			params: params,
			files: files,
			method: 'POST',
			success: callback,
			error: errback
		};
	
		ax.ext.net.curl(opts);
		
	}
	
	
	function crop(scb, ecb, imgUrl, rect) {
		var url = ax.util.format(app.def.API_CROP, encodeURIComponent(imgUrl), encodeURIComponent(rect)); // + '&' + (new Date().getTime());
		
		if (_DEBUG) { console.log('API_CROP url', url, 'rect', rect); }
		
		var errback = function(e) {
			if (_DEBUG) { console.log('API_CROP ERR', e.message); }
			if (ecb) { ecb(e); }
		};
		
		// var params = {
			// svckey: app.def.SVC_KEY,
			// fileUrl: imgUrl,
			// rect: rect,
			// faceId:0
		// };
		
		var callback = function(o) {
			try {
				if (_DEBUG) { console.log('API_CROP SUCCESS', o.data); }
				var data = app.api.validateResponse(o);
				
				if (scb) { scb.apply(app.pageSelectFace, [data.cropImg]); }

			} catch(e) {
				errback({code: '999', message: 'API_CROP validateResponse err ' + e.message});
			}
		};
	
		ax.ext.net.curl({
			method:'GET',
			url: url,
			//params: params,
			headers: app.def.API_HEADERS,
			timeout: app.def.TIMEOUT_API_DEFAULT,
			success: callback,
			error: errback
		});
	}


	var oApi = {};
	ax.def(oApi)
		.method('appNotice', appNotice)
		.method('battle', battle)
		.method('whois', whois)
		.method('upload', upload)
		.method('crop', crop)
		.method('validateResponse', validateResponse);
	
	ax.def(g.app).constant('api', oApi);

// }catch(e){
	// console.log('app_api.js def err ' + e.message);
// }
}(window));
